-- Debug functions needs to be in global scope to be able to be called from the classes

function debugLog(message)
	if false then -- set for debug
		local string = math.floor(game.tick / 60) .. " : " .. message
		game.player.print(string)
		print(string)
	end
end

--

module(..., package.seeall)
require "defines"
require "util"

require "wagons.wagon"
require "wagons.proxywagon"
require "wagons.passivewagon"
require "wagons.activewagon"
require "wagons.storagewagon"
require "wagons.requesterwagon"

-- Initialisation
game.on_init(function() oninit() end)
game.on_load(function() onload() end)

-- Entity was placed on map
game.on_event(defines.events.on_built_entity, function(event) onbuiltentity(event) end)
game.on_event(defines.events.on_robot_built_entity, function(event) onbuiltentity(event) end)

-- Entity item was removed from the map
game.on_event(defines.events.on_entity_died, function(event) onentityremoved(event) end)
game.on_event(defines.events.on_preplayer_mined_item, function(event) onentityremoved(event) end)
game.on_event(defines.events.on_robot_pre_mined, function(event) onentityremoved(event) end)

-- Main loop
game.on_event(defines.events.on_tick, function(event) ontick(event) end)

function oninit()
	debugLog("init")
	global.logisticWagons = global.logisticWagons or {}
end

function onload()
	debugLog("load")
	global.logisticWagons = global.logisticWagons or {}
	
	for i,wagon in pairs(global.logisticWagons) do	
		debugLog("Loaded wagon: " .. i .. " - " .. serpent.dump(wagon))
		global.logisticWagons[i] = _G[wagon.wagonType]:new(wagon.parent,wagon)
	end

--	This is migration data, I could not get the migration scripts to acces the global for some reason. Bug? Either way, I use class definitions and a nil check is probably as cheap as the factorio internal check
	if(global.logisticWagonsData ~= nil) then
		for i,wagon in pairs(global.logisticWagonsData) do
			if (wagon.proxy[1].valid) then
				local container = wagon.proxy[1].get_inventory(1)
				local contents = container.get_contents()
				for name,count in pairs(contents) do
					container.remove({name=name,count=count})
				end
			end
			wagon.proxy[1].destroy()
			
			on_built_entity({["name"] = wagon.wagon.name, ["created_entity"] = wagon.wagon})
			
			if(wagon.wagon.name == "lw-cargo-wagon-requester") then
				local newwagon = global.logisticWagons[#global.logisticWagons]
				newwagon.requestSlots = wagon.requestSlots
				newwagon:setRequestSlots()
			end
		end
		
		global.logisticWagonsData = nil
	end
end

function ontick(event)
--	debugLog("tick")
	for i,wagon in pairs(global.logisticWagons) do
		wagon:updateWagon(event.tick)
	end
end

function onbuiltentity(event)
	debugLog("Build event: " .. serpent.dump(event))
	local entity = event.created_entity
	if entity.name == "lw-cargo-wagon-passive" then
		table.insert(global.logisticWagons, PassiveWagon:new(entity))
	elseif entity.name == "lw-cargo-wagon-active" then
		table.insert(global.logisticWagons, ActiveWagon:new(entity))
	elseif entity.name == "lw-cargo-wagon-requester" then
		table.insert(global.logisticWagons, RequesterWagon:new(entity))
	elseif entity.name == "lw-cargo-wagon-storage" then
		table.insert(global.logisticWagons, StorageWagon:new(entity))
	end
end

function onentityremoved(event)
--	debugLog("Removal event: " .. serpent.dump(event))
	local entity = event.entity
	if isKnownEntity(entity.name) then
		debugLog("A wagon was removed!")
		local wagonPos = findWagonPosition(entity)
		if wagonPos ~= nil then
			debugLog("Wagon pos: "..wagonPos)
			local wagon = table.remove(global.logisticWagons, wagonPos)
			if(wagon.proxy) then
				wagon:removeProxy()
			end
		end
	end
end

function isKnownEntity(entity_name)
	if( global.logisticWagonsEntityNames ~= nil and global.logisticWagonsEntityNames[entity_name] ~= nil ) then
		return true
	end
end

function findWagonPosition(entity)
	for i,wagon in pairs(global.logisticWagons) do	
		if(wagon.parent == entity) then
			return i
		end
	end
	
	return nil;
end


